/*
 * ===========================================================================
 * PD4ML: HTML to PDF Converter for Java.
 * DXL-to-PDF converting agent
 * 
 * http://pd4ml.com, 2007-2008
 * ===========================================================================
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.lang.reflect.Field;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.xml.sax.SAXException;
import org.zefer.pd4ml.PD4Constants;
import org.zefer.pd4ml.PD4ML;
import org.zefer.pd4ml.PD4PageMark;

public class Dxl2Pdf {

	private final static char quote = File.pathSeparatorChar == '\\' ? '\"' : '\''; 

	private final static String USAGE = "Usage: java -Xmx512m [-Djava.awt.headless=true] Dxl2Pdf "+quote+"<dxl_path>"+quote+" <htmlWidth> [pageFormatName|WxH] [-xsl <xsl_path>]" +
	"[-pdfa] [-smarttablesplit] [-permissions <NUMBER>] [-bookmarks <HEADINGS|ANCHORS>] " +
	"[-orientation <PORTRAIT|LANDSCAPE>] [-insets <T,L,B,R,><mm|pt>] [-bgcolor <#RGB>] [-bgimage "+quote+"<url>"+quote+"] [-ttf <ttf_fonts_dir>]" + 
	"[-addstyle <CSS code>] [-pdfforms] [-protectpud] [-adjustwidth] [-fitapage] [-nohyperlinks] [-noimagesplit] " +
	"[-author <author name>] [-title <title override>] [-cookie <name> <value>] [-param <name> <value>] [-header "+quote+"<header HTML code>"+quote+"] " +
	"[-footer "+quote+"<footer HTML code>"+quote+"] [-pagerange <page>] [-out <output_file_path>] [-outformat <pdf|" + PD4Constants.RTF + "|" + PD4Constants.RTF_WMF + "] " +
	"[-password <password>] [-usetmpfiles] [-dumphtml] [-debug]";
	
	public String dxl; // input DXL location
	public String dxlsource; // input DXL source; if specified 'dxl' (DXL location) is ignored
	public StringReader in; // DXL transformed to HTML
	public String xsl = "dxl4pd4ml.xsl"; // XSL for the transformation
	public boolean dontPassAttachmentsToXalan;
	public boolean dumpHtml;
	public int width = -1;
	public String format = "A4";
	public int permissions = -1;
	public String bookmarks;
	public String orientation;
	public String insets;
	public String ttf;
	public String out;
	public boolean smart;

	public boolean forms;
	public boolean physUnits;
	public boolean adjustWidth;
	public boolean fitPage;
	public boolean hyperlinks = true;
	public boolean imgSplit = true;
	public boolean pdfa;
	public boolean debug;
	
	public String author;
	public String title;
	public String range;
	public String password;

	public String bgcolor;
	public String bgimage;

	public ArrayList styles;
	
	public HashMap cookies;
	public HashMap dynParams;
	
	public String header;
	public String footer;

	private static String outputFormat;

	public Dxl2Pdf() {
	}
	
	public static void main(String[] args) throws Exception {
		Dxl2Pdf converter = new Dxl2Pdf();
		
		boolean success = converter.readParameters(args);
		if ( !success ) {
			System.exit(1);
		}
		
		if (converter.out != null) {
			OutputStream os = new FileOutputStream(converter.out);
			converter.generatePDF( os );
			os.close();
		} else {
			converter.generatePDF( System.out );
		}
	}
	
	private void generatePDF( OutputStream outputStream ) throws Exception {

		String html = transformXML();
		if ( dumpHtml ) {
			System.out.println( html );
		}

//		File fx = new File( "c:/tmp.htm" );
//		FileOutputStream fos = new FileOutputStream(fx);
//		PrintStream ps = new PrintStream(fos);
//		ps.println( html );
//		ps.close();
		
		in = new StringReader( html );
		
		PD4ML pd4ml = new PD4ML();

		if (insets != null) {
			StringTokenizer st = new StringTokenizer(insets, ",");
			try {
				int top = Integer.parseInt(st.nextToken());
				int left = Integer.parseInt(st.nextToken());
				int bottom = Integer.parseInt(st.nextToken());
				int right = Integer.parseInt(st.nextToken());
				String units = st.nextToken();
				Insets ins = new Insets(top, left, bottom, right);
				if ("mm".equalsIgnoreCase(units)) {
					pd4ml.setPageInsetsMM(ins);
				} else {
					pd4ml.setPageInsets(ins);
				}
			} catch (Exception e) {
				throw new Exception(
						"Invalid page insets (top, left, bottom, right, units): "
								+ insets);
			}
		}

		pd4ml.setHtmlWidth(width);

		try {
			Class c = PD4Constants.class;
			Field f = c.getField( format );
			Dimension d = (Dimension) f.get(pd4ml);

			if (orientation != null && "LANDSCAPE".equalsIgnoreCase(orientation)) {
				d = pd4ml.changePageOrientation(d);
			}

			pd4ml.setPageSize(d);
		} catch (Exception e) {
			StringTokenizer st = new StringTokenizer( format, "x");
			try {
				int width = Integer.parseInt(st.nextToken());
				int height = Integer.parseInt(st.nextToken());
				if ( height < 1 || width < 1 ) {
					throw new Exception("nop");
				}
				
				Dimension d = new Dimension(width,height);
				if (orientation != null && "LANDSCAPE".equalsIgnoreCase(orientation)) {
					d = pd4ml.changePageOrientation(d);
				}

				pd4ml.setPageSize(d);
			} catch (Exception ex) {
				throw new Exception( "Invalid page format: " + format );
			}
		}

		pd4ml.enableSmartTableBreaks(smart);

		if (permissions != -1) {
			if ( password != null ) {
				pd4ml.setPermissions(password, permissions, true);
			} else {
				pd4ml.setPermissions("empty", permissions, true);
			}
		} else {
			if ( password != null ) {
				pd4ml.setPermissions(password, 0xfffffffc, true);
			}
		}

		if (bookmarks != null) {
			if ("ANCHORS".equalsIgnoreCase(bookmarks)) {
				pd4ml.generateOutlines(false);
			} else if ("HEADINGS".equalsIgnoreCase(bookmarks)) {
				pd4ml.generateOutlines(true);
			}
		}

		if (ttf != null && ttf.length() > 0) {
			pd4ml.useTTF(ttf, true);
		}

		if (forms) {
			pd4ml.generatePdfForms(true, "Arial");
		}

		if (physUnits) {
			pd4ml.protectPhysicalUnitDimensions();
		}

		if (adjustWidth) {
			pd4ml.adjustHtmlWidth();
		}

		if (fitPage) {
			pd4ml.fitPageVertically();
		}

		if (!hyperlinks) {
			pd4ml.disableHyperlinks();
		}

		if (!imgSplit) {
			pd4ml.enableImgSplit(false);
		}

		if (author != null ) {
			pd4ml.setAuthorName(author);
		}

		if (title != null ) {
			pd4ml.setDocumentTitle(title);
		}

		if (range != null ) {
			pd4ml.outputRange(range);
		}

		if (dynParams != null ) {
			pd4ml.setDynamicParams(dynParams);
		}

		if (styles != null ) {
			Iterator ii = styles.iterator();
			while (ii.hasNext()) {
				String style = (String)ii.next();
				pd4ml.addStyle(style, true);
			}
		}

		if (cookies != null ) {
			Iterator ii = cookies.keySet().iterator();
			while (ii.hasNext()) {
				String key = (String)ii.next();
				String value = (String)cookies.get(key);
				pd4ml.setCookie(key, value);
			}
		}

		if (pdfa) {
			pd4ml.generatePdfa(true);
		}

		if ( outputFormat != null && (PD4Constants.RTF.equalsIgnoreCase(outputFormat) || PD4Constants.RTF_WMF.equalsIgnoreCase(outputFormat)) ) {
			pd4ml.outputFormat(outputFormat);
		}

		if ( footer != null && footer.trim().length() > 0 || 
				bgimage != null && bgimage.trim().length() > 0 || 
				bgcolor != null && bgcolor.trim().length() > 0  ) {
			
			PD4PageMark footerObj = new PD4PageMark();
			footerObj.setAreaHeight(-1); // autocompute
			try {
				if ( footer != null && footer.trim().length() > 0 ) {
					footerObj.setHtmlTemplate(footer);
				}
				
				if ( bgimage != null && bgimage.trim().length() > 0 ) {
					footerObj.setPageBackgroundImageUrl(bgimage);
				}
				
				if ( bgcolor != null && bgcolor.trim().length() > 0 ) {
					Color c = Color.decode(bgcolor);
					footerObj.setPageBackgroundColor(c);
				}
				
				pd4ml.setPageFooter(footerObj);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		if ( header != null && header.trim().length() > 0 ) {
			PD4PageMark headerObj = new PD4PageMark();
			headerObj.setAreaHeight(-1); // autocompute
			headerObj.setHtmlTemplate(header);
			pd4ml.setPageHeader(headerObj);
		}

		if (debug) {
			pd4ml.enableDebugInfo();
		}

		long start = System.currentTimeMillis();
		if ( debug ) {
			System.out.println( "HTML->PDF conversion... " );
		}
		pd4ml.render(in, outputStream);
	}
	
	public boolean readParameters( String[] args ) {

		boolean formatSet = false;
		
		for( int i = 0; i < args.length; i++ ) {

			if ( "-usetmpfiles".equalsIgnoreCase( args[i] ) ) {
				dontPassAttachmentsToXalan = true;
				continue;
			}

			if ( "-dumphtml".equalsIgnoreCase( args[i] ) ) {
				dumpHtml = true;
				continue;
			}

			if ( "-pdfforms".equalsIgnoreCase( args[i] ) ) {
				forms = true;
				continue;
			}

			if ( "-protectpud".equalsIgnoreCase( args[i] ) ) {
				physUnits = true;
				continue;
			}

			if ( "-adjustwidth".equalsIgnoreCase( args[i] ) ) {
				adjustWidth = true;
				continue;
			}

			if ( "-fitapage".equalsIgnoreCase( args[i] ) ) {
				fitPage = true;
				continue;
			}

			if ( "-nohyperlinks".equalsIgnoreCase( args[i] ) ) {
				hyperlinks = false;
				continue;
			}

			if ( "-pdfa".equalsIgnoreCase( args[i] ) ) {
				pdfa = true;
				continue;
			}

			if ( "-noimagesplit".equalsIgnoreCase( args[i] ) ) {
				imgSplit = false;
				continue;
			}

			if ( "-debug".equalsIgnoreCase( args[i] ) ) {
				debug = true;
				continue;
			}

			if ( "-smarttablesplit".equalsIgnoreCase( args[i] ) ) {
				smart = true;
				continue;
			}

			if ( "-bgcolor".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: color is missing (Example: -bgcolor "+quote+"#ffffcc"+quote+" or  -bgcolor 0xffffcc)");
					System.out.println( USAGE );
					return false; 
				}
				bgcolor = args[i];
				continue;
			}
			
			if ( "-bgimage".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: author name is missing (Example: -bgimage "+quote+"http://pd4ml.com/i/blank.jpg"+quote+" or -bgimage "+quote+"file:/resources/images/blank.jpg"+quote+")");
					System.out.println( USAGE );
					return false; 
				}
				bgimage = args[i];
				continue;
			}
			
			if ( "-author".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: author name is missing");
					System.out.println( USAGE );
					return false; 
				}
				author = args[i];
				continue;
			}

			if ( "-xsl".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: xsl location is missing");
					System.out.println( USAGE );
					return false; 
				}
				xsl = args[i];
				continue;
			}

			if ( "-title".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: title override string is missing");
					System.out.println( USAGE );
					return false; 
				}
				title = args[i];
				continue;
			}

			if ( "-password".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: password string is missing");
					System.out.println( USAGE );
					return false; 
				}
				password = args[i];
				continue;
			}

			if ( "-pagerange".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: no output page range definition string (Example: -pagerange "+quote+"2-3,7+"+quote+")");
					System.out.println( USAGE );
					return false; 
				}
				range = args[i];
				continue;
			}

			if ( "-header".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: PDF header definition HTML code is missing (Example: -header "+quote+"<div width=100% align=right>$[page] of $[total]</div>"+quote+")");
					System.out.println( USAGE );
					return false; 
				}
				header = args[i];
				continue;
			}

			if ( "-footer".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: PDF footer definition HTML code is missing (Example: -footer "+quote+"<div width=100% align=right>$[page] of $[total]</div>"+quote+")");
					System.out.println( USAGE );
					return false; 
				}
				footer = args[i];
				continue;
			}

			if ( "-addstyle".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: style definition is missing  (Example: -addstyle "+quote+"IMG, TR {page-break-inside: avoid}"+quote+")");
					System.out.println( USAGE );
					return false; 
				}
				
				if ( styles == null ) {
					styles = new ArrayList();
				}
				
				String style = args[i];
				styles.add(style);
				continue;
			}

			if ( "-cookie".equals( args[i] ) ) {
				++i;
				if ( i+1 == args.length ) {
					System.out.println("invalid parameter: incomplete cookie data  (Example: -cookie JSESSIONID "+quote+"9034657927465;path=/"+quote+")");
					System.out.println( USAGE );
					return false; 
				}
				
				if ( cookies == null ) {
					cookies = new HashMap();
				}
				
				String key = args[i];
				String value = args[++i];
				
				cookies.put(key, value);
				continue;
			}

			if ( "-param".equals( args[i] ) ) {
				++i;
				if ( i+1 == args.length ) {
					System.out.println("invalid parameter: incomplete cookie data  (Example: -param pd4ml.basic.authentication "+quote+"user:password"+quote+")");
					System.out.println( USAGE );
					return false; 
				}
				
				if ( dynParams == null ) {
					dynParams = new HashMap();
				}
				
				String key = args[i];
				String value = args[++i];
				
				dynParams.put(key, value);
				continue;
			}

			if ( "-permissions".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: missing permissions number (a sum of single permission codes)");
					System.out.println( USAGE );
					return false; 
				}
				permissions = Integer.parseInt(args[i]);
				continue;
			}

			if ( "-bookmarks".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: missing bookmark type (HEADINGS or ANCHORS)");
					System.out.println( USAGE );
					return false; 
				}
				bookmarks = args[i];
				continue;
			}

			if ( "-orientation".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: missing orientation type (PORTRAIT or LANDSCAPE)");
					System.out.println( USAGE );
					return false; 
				}
				orientation = args[i];
				continue;
			}

			if ( "-outformat".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: missing format type (pdf OR " + PD4Constants.RTF + " OR " + PD4Constants.RTF_WMF + ")");
					System.out.println( USAGE );
					return false; 
				}
				outputFormat = args[i];
				continue;
			}

			if ( "-insets".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: missing insets (for exampe: -insets 5,10,5,5mm)");
					System.out.println( USAGE );
					return false; 
				}
				insets = args[i];
				continue;
			}

			if ( "-ttf".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: missing TTF fonts directory");
					System.out.println( USAGE );
					return false; 
				}
				ttf = args[i];
				continue;
			}

			if ( "-out".equals( args[i] ) ) {
				++i;
				if ( i == args.length ) {
					System.out.println("invalid parameter: missing output file name");
					System.out.println( USAGE );
					return false; 
				}
				out = args[i];
				continue;
			}

			if ( args[i].startsWith("-")  ) {
				System.out.println("unknown parameter: " + args[i] );
				System.out.println( USAGE );
				return false; 
			}

			if ( dxl == null ) {
				dxl = args[i];
				continue;
			}

			if ( width == -1 ) {
				width = Integer.parseInt(args[i]);
				continue;
			}

			if ( !formatSet ) {
				formatSet = true;
				format = args[i];
				continue;
			}

			System.out.println("unexpected parameter: " + args[i]);
			System.out.println( USAGE );
			return false; 
		}
		
		if ( dxl == null ) {
			System.out.println("source DXL is missing");
			System.out.println( USAGE );
			return false; 
		}

		if ( width == -1 ) {
			System.out.println("HTML width parameter is missing");
			System.out.println( USAGE );
			return false; 
		}
		
		if ( out == null ) {
			debug = false;
			dumpHtml = false;
		}
		
		return true;
	}
	
	/* --- Utility methods --- */
	
	public static final String extractAttachments( String body, boolean debug ) throws IOException {

		if (body != null) {

			int index01 = body.indexOf( "<filedata" );
			
			if ( index01 < 0 ) {
				return body;
			}
			
			StringBuffer sb = new StringBuffer();
			
			int index02 = 0;
			while (index01 != -1) {
				String portion = body.substring( index02, index01 );
				sb.append( portion );
				sb.append( "<tmpfilereference>" );
				
				index02 = body.indexOf( "</filedata>", index01 + 1 );

				String fileName = "bin";
				int name01 = body.lastIndexOf("name='", index02);
				if ( name01 > 0 ) {
					name01 += 6;
					int name02 = body.indexOf('\'', name01);
					if ( name02 > 0 ) {
						fileName = body.substring(name01,name02); 
					}
				}
				
				int endtag = body.indexOf( ">", index01 + 1 );
				String fileContent = body.substring( endtag + 1, index02 );
				byte[] bytes = Base64.decode( fileContent );
				
				File file = File.createTempFile("attachment", "." + fileName);
				file.deleteOnExit();
				FileOutputStream fos = new FileOutputStream(file);
				fos.write(bytes);

				String tmpFileName = file.toURI().toString();
				
				sb.append( tmpFileName );
				sb.append( "</tmpfilereference>" );
				
				index02 += 11;
				index01 = body.indexOf( "<filedata", index02 + 1 );
			}

			sb.append( body.substring( index02 ) );

			return sb.toString();
		}
		return null;
	}
	
	public String transformXML()
			throws SAXException, MalformedURLException, IOException {

		try {
			String stylesheet = readFile( xsl );
			String xml = dxlsource;
			if ( xml == null ) {
				xml = readFile( dxl );
			}
			
			if ( dontPassAttachmentsToXalan ) {
				long start = System.currentTimeMillis();
				if ( debug ) {
					System.out.println( "DXL attachment extraction... " );
				}
				xml = extractAttachments(xml, debug);
				long duration = System.currentTimeMillis() - start;
				if ( debug ) {
					System.out.println( "done in " + duration + "ms" );
					
					System.out.println( xml );
					System.out.println( "--------------------" );
				}
			}
			
			java.io.ByteArrayOutputStream baos = new java.io.ByteArrayOutputStream();

//			javax.xml.transform.Source xmlSource = new javax.xml.transform.stream.StreamSource(new StringReader(xml));
//			javax.xml.transform.Source xsltSource = new javax.xml.transform.stream.StreamSource(new StringReader(xsl));
			javax.xml.transform.Result result = new javax.xml.transform.stream.StreamResult(new OutputStreamWriter(baos, "UTF-8"));

			javax.xml.parsers.SAXParserFactory spf = javax.xml.parsers.SAXParserFactory.newInstance();
			spf.setNamespaceAware(true);
			spf.setValidating(false); // Turn off validation
			spf.setFeature("http://xml.org/sax/features/validation", false);
			spf.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);

			org.xml.sax.XMLReader reader = spf.newSAXParser().getXMLReader();
			javax.xml.transform.Source xmlSource = new javax.xml.transform.sax.SAXSource(reader,new org.xml.sax.InputSource(new StringReader(xml)));

			org.xml.sax.XMLReader reader2 = spf.newSAXParser().getXMLReader();
			javax.xml.transform.Source xsltSource = new javax.xml.transform.sax.SAXSource(reader2,new org.xml.sax.InputSource(new StringReader(stylesheet)));
			
			javax.xml.transform.TransformerFactory transFact = javax.xml.transform.TransformerFactory.newInstance();
			
//			R8.5 workaround: http://code.google.com/p/googleappengine/issues/detail?id=1452			
//			javax.xml.transform.TransformerFactory transFact = 
//				javax.xml.transform.TransformerFactory.newInstance("org.apache.xalan.processor.TransformerFactoryImpl", null); 

			javax.xml.transform.Transformer trans = transFact.newTransformer(xsltSource);

			long start = System.currentTimeMillis();
			if ( debug ) {
				System.out.println( "DXL->HTML transformation... " );
			}
			trans.transform(xmlSource, result);
			long duration = System.currentTimeMillis() - start;
			if ( debug ) {
				System.out.println( "done in " + duration + "ms" );
			}

			return baos.toString("utf-8");
		} catch (javax.xml.transform.TransformerException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}
	
	private final static String readFile( String path ) throws Exception {

		File f = new File( path );
		FileInputStream is = new FileInputStream(f);
		BufferedInputStream bis = new BufferedInputStream(is);
		
		ByteArrayOutputStream fos = new ByteArrayOutputStream();
		byte buffer[] = new byte[2048];

		int read;
		do {
			read = is.read(buffer, 0, buffer.length);
			if (read > 0) { // something to put down
				fos.write(buffer, 0, read); 
			}
		} while (read > -1);

		fos.close();
		bis.close();
		is.close();

		return fos.toString("UTF-8");
	}
	/**
	 * Encodes and decodes to and from Base64 notation.
	 *
	 * <p>
	 * Change Log:
	 * </p>
	 * <ul>
	 *  <li>v2.1 - Cleaned up javadoc comments and unused variables and methods. Added
	 *   some convenience methods for reading and writing to and from files.</li>
	 *  <li>v2.0.2 - Now specifies UTF-8 encoding in places where the code fails on systems
	 *   with other encodings (like EBCDIC).</li>
	 *  <li>v2.0.1 - Fixed an error when decoding a single byte, that is, when the
	 *   encoded data was a single byte.</li>
	 *  <li>v2.0 - I got rid of methods that used booleans to set options. 
	 *   Now everything is more consolidated and cleaner. The code now detects
	 *   when data that's being decoded is gzip-compressed and will decompress it
	 *   automatically. Generally things are cleaner. You'll probably have to
	 *   change some method calls that you were making to support the new
	 *   options format (<tt>int</tt>s that you "OR" together).</li>
	 *  <li>v1.5.1 - Fixed bug when decompressing and decoding to a             
	 *   byte[] using <tt>decode( String s, boolean gzipCompressed )</tt>.      
	 *   Added the ability to "suspend" encoding in the Output Stream so        
	 *   you can turn on and off the encoding if you need to embed base64       
	 *   data in an otherwise "normal" stream (like an XML file).</li>  
	 *  <li>v1.5 - Output stream pases on flush() command but doesn't do anything itself.
	 *      This helps when using GZIP streams.
	 *      Added the ability to GZip-compress objects before encoding them.</li>
	 *  <li>v1.4 - Added helper methods to read/write files.</li>
	 *  <li>v1.3.6 - Fixed OutputStream.flush() so that 'position' is reset.</li>
	 *  <li>v1.3.5 - Added flag to turn on and off line breaks. Fixed bug in input stream
	 *      where last buffer being read, if not completely full, was not returned.</li>
	 *  <li>v1.3.4 - Fixed when "improperly padded stream" error was thrown at the wrong time.</li>
	 *  <li>v1.3.3 - Fixed I/O streams which were totally messed up.</li>
	 * </ul>
	 *
	 * <p>
	 * I am placing this code in the Public Domain. Do with it as you will.
	 * This software comes with no guarantees or warranties but with
	 * plenty of well-wishing instead!
	 * Please visit <a href="http://iharder.net/base64">http://iharder.net/base64</a>
	 * periodically to check for updates or to contribute improvements.
	 * </p>
	 *
	 * @author Robert Harder
	 * @author rob@iharder.net
	 * @version 2.1
	 */

	public static class Base64
	{
	    
	/* ********  P U B L I C   F I E L D S  ******** */   
	    
	    /** No options specified. Value is zero. */
	    public final static int NO_OPTIONS = 0;
	    
	    /** Specify encoding. */
	    public final static int ENCODE = 1;
	    
	    
	    /** Specify decoding. */
	    public final static int DECODE = 0;
	    
	    
	    /** Specify that data should be gzip-compressed. */
	    public final static int GZIP = 2;
	    
	    
	    /** Don't break lines when encoding (violates strict Base64 specification) */
	    public final static int DONT_BREAK_LINES = 8;
	    
	    
	/* ********  P R I V A T E   F I E L D S  ******** */  
	    
	    /** The equals sign (=) as a byte. */
	    private final static byte EQUALS_SIGN = (byte)'=';
	    
	    
	    /** Preferred encoding. */
	    private final static String PREFERRED_ENCODING = "UTF-8";
	    
	    
	    /** The 64 valid Base64 values. */
	    private final static byte[] ALPHABET;
	    private final static byte[] _NATIVE_ALPHABET = /* May be something funny like EBCDIC */
	    {
	        (byte)'A', (byte)'B', (byte)'C', (byte)'D', (byte)'E', (byte)'F', (byte)'G',
	        (byte)'H', (byte)'I', (byte)'J', (byte)'K', (byte)'L', (byte)'M', (byte)'N',
	        (byte)'O', (byte)'P', (byte)'Q', (byte)'R', (byte)'S', (byte)'T', (byte)'U', 
	        (byte)'V', (byte)'W', (byte)'X', (byte)'Y', (byte)'Z',
	        (byte)'a', (byte)'b', (byte)'c', (byte)'d', (byte)'e', (byte)'f', (byte)'g',
	        (byte)'h', (byte)'i', (byte)'j', (byte)'k', (byte)'l', (byte)'m', (byte)'n',
	        (byte)'o', (byte)'p', (byte)'q', (byte)'r', (byte)'s', (byte)'t', (byte)'u', 
	        (byte)'v', (byte)'w', (byte)'x', (byte)'y', (byte)'z',
	        (byte)'0', (byte)'1', (byte)'2', (byte)'3', (byte)'4', (byte)'5', 
	        (byte)'6', (byte)'7', (byte)'8', (byte)'9', (byte)'+', (byte)'/'
	    };
	    
	    /** Determine which ALPHABET to use. */
	    static
	    {
	        byte[] __bytes;
	        try
	        {
	            __bytes = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".getBytes( PREFERRED_ENCODING );
	        }   // end try
	        catch (java.io.UnsupportedEncodingException use)
	        {
	            __bytes = _NATIVE_ALPHABET; // Fall back to native encoding
	        }   // end catch
	        ALPHABET = __bytes;
	    }   // end static
	    
	    
	    /** 
	     * Translates a Base64 value to either its 6-bit reconstruction value
	     * or a negative number indicating some other meaning.
	     **/
	    private final static byte[] DECODABET =
	    {   
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,                 // Decimal  0 -  8
	        -5,-5,                                      // Whitespace: Tab and Linefeed
	        -9,-9,                                      // Decimal 11 - 12
	        -5,                                         // Whitespace: Carriage Return
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 14 - 26
	        -9,-9,-9,-9,-9,                             // Decimal 27 - 31
	        -5,                                         // Whitespace: Space
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,              // Decimal 33 - 42
	        62,                                         // Plus sign at decimal 43
	        -9,-9,-9,                                   // Decimal 44 - 46
	        63,                                         // Slash at decimal 47
	        52,53,54,55,56,57,58,59,60,61,              // Numbers zero through nine
	        -9,-9,-9,                                   // Decimal 58 - 60
	        -1,                                         // Equals sign at decimal 61
	        -9,-9,-9,                                      // Decimal 62 - 64
	        0,1,2,3,4,5,6,7,8,9,10,11,12,13,            // Letters 'A' through 'N'
	        14,15,16,17,18,19,20,21,22,23,24,25,        // Letters 'O' through 'Z'
	        -9,-9,-9,-9,-9,-9,                          // Decimal 91 - 96
	        26,27,28,29,30,31,32,33,34,35,36,37,38,     // Letters 'a' through 'm'
	        39,40,41,42,43,44,45,46,47,48,49,50,51,     // Letters 'n' through 'z'
	        -9,-9,-9,-9                                 // Decimal 123 - 126
	        /*,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 127 - 139
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 140 - 152
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 153 - 165
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 166 - 178
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 179 - 191
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 192 - 204
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 205 - 217
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 218 - 230
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,     // Decimal 231 - 243
	        -9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9         // Decimal 244 - 255 */
	    };
	    
	    // I think I end up not using the BAD_ENCODING indicator.
	    //private final static byte BAD_ENCODING    = -9; // Indicates error in encoding
	    private final static byte WHITE_SPACE_ENC = -5; // Indicates white space in encoding
	    private final static byte EQUALS_SIGN_ENC = -1; // Indicates equals sign in encoding

	    
	    /** Defeats instantiation. */
	    private Base64(){}
	    
	    
	    
	/* ********  D E C O D I N G   M E T H O D S  ******** */
	    
	    
	    /**
	     * Decodes four bytes from array <var>source</var>
	     * and writes the resulting bytes (up to three of them)
	     * to <var>destination</var>.
	     * The source and destination arrays can be manipulated
	     * anywhere along their length by specifying 
	     * <var>srcOffset</var> and <var>destOffset</var>.
	     * This method does not check to make sure your arrays
	     * are large enough to accomodate <var>srcOffset</var> + 4 for
	     * the <var>source</var> array or <var>destOffset</var> + 3 for
	     * the <var>destination</var> array.
	     * This method returns the actual number of bytes that 
	     * were converted from the Base64 encoding.
	     * 
	     *
	     * @param source the array to convert
	     * @param srcOffset the index where conversion begins
	     * @param destination the array to hold the conversion
	     * @param destOffset the index where output will be put
	     * @return the number of decoded bytes converted
	     * @since 1.3
	     */
	    private static int decode4to3( byte[] source, int srcOffset, byte[] destination, int destOffset )
	    {
	        // Example: Dk==
	        if( source[ srcOffset + 2] == EQUALS_SIGN )
	        {
	            // Two ways to do the same thing. Don't know which way I like best.
	            //int outBuff =   ( ( DECODABET[ source[ srcOffset    ] ] << 24 ) >>>  6 )
	            //              | ( ( DECODABET[ source[ srcOffset + 1] ] << 24 ) >>> 12 );
	            int outBuff =   ( ( DECODABET[ source[ srcOffset    ] ] & 0xFF ) << 18 )
	                          | ( ( DECODABET[ source[ srcOffset + 1] ] & 0xFF ) << 12 );
	            
	            destination[ destOffset ] = (byte)( outBuff >>> 16 );
	            return 1;
	        }
	        
	        // Example: DkL=
	        else if( source[ srcOffset + 3 ] == EQUALS_SIGN )
	        {
	            // Two ways to do the same thing. Don't know which way I like best.
	            //int outBuff =   ( ( DECODABET[ source[ srcOffset     ] ] << 24 ) >>>  6 )
	            //              | ( ( DECODABET[ source[ srcOffset + 1 ] ] << 24 ) >>> 12 )
	            //              | ( ( DECODABET[ source[ srcOffset + 2 ] ] << 24 ) >>> 18 );
	            int outBuff =   ( ( DECODABET[ source[ srcOffset     ] ] & 0xFF ) << 18 )
	                          | ( ( DECODABET[ source[ srcOffset + 1 ] ] & 0xFF ) << 12 )
	                          | ( ( DECODABET[ source[ srcOffset + 2 ] ] & 0xFF ) <<  6 );
	            
	            destination[ destOffset     ] = (byte)( outBuff >>> 16 );
	            destination[ destOffset + 1 ] = (byte)( outBuff >>>  8 );
	            return 2;
	        }
	        
	        // Example: DkLE
	        else
	        {
	            try{
	            // Two ways to do the same thing. Don't know which way I like best.
	            //int outBuff =   ( ( DECODABET[ source[ srcOffset     ] ] << 24 ) >>>  6 )
	            //              | ( ( DECODABET[ source[ srcOffset + 1 ] ] << 24 ) >>> 12 )
	            //              | ( ( DECODABET[ source[ srcOffset + 2 ] ] << 24 ) >>> 18 )
	            //              | ( ( DECODABET[ source[ srcOffset + 3 ] ] << 24 ) >>> 24 );
	            int outBuff =   ( ( DECODABET[ source[ srcOffset     ] ] & 0xFF ) << 18 )
	                          | ( ( DECODABET[ source[ srcOffset + 1 ] ] & 0xFF ) << 12 )
	                          | ( ( DECODABET[ source[ srcOffset + 2 ] ] & 0xFF ) <<  6)
	                          | ( ( DECODABET[ source[ srcOffset + 3 ] ] & 0xFF )      );

	            
	            destination[ destOffset     ] = (byte)( outBuff >> 16 );
	            destination[ destOffset + 1 ] = (byte)( outBuff >>  8 );
	            destination[ destOffset + 2 ] = (byte)( outBuff       );

	            return 3;
	            }catch( Exception e){
	                System.out.println(""+source[srcOffset]+ ": " + ( DECODABET[ source[ srcOffset     ] ]  ) );
	                System.out.println(""+source[srcOffset+1]+  ": " + ( DECODABET[ source[ srcOffset + 1 ] ]  ) );
	                System.out.println(""+source[srcOffset+2]+  ": " + ( DECODABET[ source[ srcOffset + 2 ] ]  ) );
	                System.out.println(""+source[srcOffset+3]+  ": " + ( DECODABET[ source[ srcOffset + 3 ] ]  ) );
	                return -1;
	            }   //e nd catch
	        }
	    }   // end decodeToBytes
	    
	    
	    
	    
	    /**
	     * Very low-level access to decoding ASCII characters in
	     * the form of a byte array. Does not support automatically
	     * gunzipping or any other "fancy" features.
	     *
	     * @param source The Base64 encoded data
	     * @param off    The offset of where to begin decoding
	     * @param len    The length of characters to decode
	     * @return decoded data
	     * @since 1.3
	     */
	    public static byte[] decode( byte[] source, int off, int len )
	    {
	        int    len34   = len * 3 / 4;
	        byte[] outBuff = new byte[ len34 ]; // Upper limit on size of output
	        int    outBuffPosn = 0;
	        
	        byte[] b4        = new byte[4];
	        int    b4Posn    = 0;
	        int    i         = 0;
	        byte   sbiCrop   = 0;
	        byte   sbiDecode = 0;
	        for( i = off; i < off+len; i++ )
	        {
	            sbiCrop = (byte)(source[i] & 0x7f); // Only the low seven bits
	            sbiDecode = DECODABET[ sbiCrop ];
	            
	            if( sbiDecode >= WHITE_SPACE_ENC ) // White space, Equals sign or better
	            {
	                if( sbiDecode >= EQUALS_SIGN_ENC )
	                {
	                    b4[ b4Posn++ ] = sbiCrop;
	                    if( b4Posn > 3 )
	                    {
	                        outBuffPosn += decode4to3( b4, 0, outBuff, outBuffPosn );
	                        b4Posn = 0;
	                        
	                        // If that was the equals sign, break out of 'for' loop
	                        if( sbiCrop == EQUALS_SIGN )
	                            break;
	                    }   // end if: quartet built
	                    
	                }   // end if: equals sign or better
	                
	            }   // end if: white space, equals sign or better
	            else
	            {
	                System.err.println( "Bad Base64 input character at " + i + ": " + source[i] + "(decimal)" );
	                return null;
	            }   // end else: 
	        }   // each input character
	                                   
	        byte[] out = new byte[ outBuffPosn ];
	        System.arraycopy( outBuff, 0, out, 0, outBuffPosn ); 
	        return out;
	    }   // end decode
	    
	    
	    
	    
	    /**
	     * Decodes data from Base64 notation, automatically
	     * detecting gzip-compressed data and decompressing it.
	     *
	     * @param s the string to decode
	     * @return the decoded data
	     * @since 1.4
	     */
	    public static byte[] decode( String s )
	    {   
	        byte[] bytes;
	        try
	        {
	            bytes = s.getBytes( PREFERRED_ENCODING );
	        }   // end try
	        catch( java.io.UnsupportedEncodingException uee )
	        {
	            bytes = s.getBytes();
	        }   // end catch
			//</change>
	        
	        // Decode
	        bytes = decode( bytes, 0, bytes.length );
	        
	        
	        // Check to see if it's gzip-compressed
	        // GZIP Magic Two-Byte Number: 0x8b1f (35615)
	        if( bytes != null && bytes.length >= 4 )
	        {
	            
	            int head = ((int)bytes[0] & 0xff) | ((bytes[1] << 8) & 0xff00);       
	            if( java.util.zip.GZIPInputStream.GZIP_MAGIC == head ) 
	            {
	                java.io.ByteArrayInputStream  bais = null;
	                java.util.zip.GZIPInputStream gzis = null;
	                java.io.ByteArrayOutputStream baos = null;
	                byte[] buffer = new byte[2048];
	                int    length = 0;

	                try
	                {
	                    baos = new java.io.ByteArrayOutputStream();
	                    bais = new java.io.ByteArrayInputStream( bytes );
	                    gzis = new java.util.zip.GZIPInputStream( bais );

	                    while( ( length = gzis.read( buffer ) ) >= 0 )
	                    {
	                        baos.write(buffer,0,length);
	                    }   // end while: reading input

	                    // No error? Get new bytes.
	                    bytes = baos.toByteArray();

	                }   // end try
	                catch( java.io.IOException e )
	                {
	                    // Just return originally-decoded bytes
	                }   // end catch
	                finally
	                {
	                    try{ baos.close(); } catch( Exception e ){}
	                    try{ gzis.close(); } catch( Exception e ){}
	                    try{ bais.close(); } catch( Exception e ){}
	                }   // end finally

	            }   // end if: gzipped
	        }   // end if: bytes.length >= 2
	        
	        return bytes;
	    }   // end decode

	}   // end class Base64	
}
